@@ -8,14 +8,17 @@ from typing import (
     Callable,
     Dict,
     FrozenSet,
+    Iterator,
     List,
     Mapping,
     Optional,
     Sequence,
     Set,
     Tuple,
+    Union,
 )
 
+import dateutil
 from dagster import (
     AssetKey,
     AssetsDefinition,
@@ -23,6 +26,7 @@ from dagster import (
     In,
     MetadataValue,
     Nothing,
+    OpExecutionContext,
     Out,
     PartitionsDefinition,
     TableColumn,
@@ -33,18 +37,29 @@ from dagster import (
 )
 from dagster._config.field import Field
 from dagster._config.field_utils import Permissive
-from dagster._core.definitions.events import CoercibleToAssetKeyPrefix
+from dagster._core.definitions.events import (
+    AssetMaterialization,
+    AssetObservation,
+    CoercibleToAssetKeyPrefix,
+    Output,
+)
 from dagster._core.definitions.load_assets_from_modules import prefix_assets
-from dagster._core.definitions.metadata import RawMetadataValue
+from dagster._core.definitions.metadata import MetadataUserInput, RawMetadataValue
 from dagster._core.errors import DagsterInvalidSubsetError
-from dagster._legacy import OpExecutionContext
 from dagster._utils.backcompat import experimental_arg_warning
-from dagster._utils.merger import deep_merge_dicts
+from dagster._utils.merger import deep_merge_dicts, merge_dicts
 
+from dagster_dbt.cli.resources import DbtCliResource
 from dagster_dbt.cli.types import DbtCliOutput
 from dagster_dbt.cli.utils import execute_cli
 from dagster_dbt.types import DbtOutput
-from dagster_dbt.utils import _get_input_name, _get_output_name, result_to_events
+from dagster_dbt.utils import (
+    ASSET_RESOURCE_TYPES,
+    _get_input_name,
+    _get_output_name,
+    result_to_events,
+    select_unique_ids_from_manifest,
+)
 
 
 def _load_manifest_for_project(
@@ -78,65 +93,21 @@ def _load_manifest_for_project(
         return json.load(f), cli_output
 
 
-def _select_unique_ids_from_manifest_json(
-    manifest_json: Mapping[str, Any], select: str, exclude: str
-) -> AbstractSet[str]:
-    """Method to apply a selection string to an existing manifest.json file."""
-    try:
-        import dbt.graph.cli as graph_cli
-        import dbt.graph.selector as graph_selector
-        from dbt.contracts.graph.manifest import Manifest
-        from dbt.graph import SelectionSpec
-        from networkx import DiGraph
-    except ImportError as e:
-        raise check.CheckError(
-            "In order to use the `select` argument on load_assets_from_dbt_manifest, you must have"
-            "`dbt-core >= 1.0.0` and `networkx` installed."
-        ) from e
+def _can_stream_events(dbt_resource: DbtCliResource) -> bool:
+    """Check if the installed dbt version supports streaming events."""
+    import dbt.version
+    from packaging import version
 
-    class _DictShim(dict):
-        """Shim to enable hydrating a dictionary into a dot-accessible object."""
-
-        def __getattr__(self, item):
-            ret = super().get(item)
-            # allow recursive access e.g. foo.bar.baz
-            return _DictShim(ret) if isinstance(ret, dict) else ret
-
-    # generate a dbt-compatible graph from the existing child map
-    graph = graph_selector.Graph(DiGraph(incoming_graph_data=manifest_json["child_map"]))
-    manifest = Manifest(
-        # dbt expects dataclasses that can be accessed with dot notation, not bare dictionaries
-        nodes={unique_id: _DictShim(info) for unique_id, info in manifest_json["nodes"].items()},  # type: ignore
-        sources={
-            unique_id: _DictShim(info) for unique_id, info in manifest_json["sources"].items()  # type: ignore
-        },
-        metrics={
-            unique_id: _DictShim(info) for unique_id, info in manifest_json["metrics"].items()  # type: ignore
-        },
-        exposures={
-            unique_id: _DictShim(info) for unique_id, info in manifest_json["exposures"].items()  # type: ignore
-        },
+    return (
+        version.parse(dbt.version.__version__) >= version.parse("1.4.0")
+        and dbt_resource._json_log_format  # noqa: SLF001
     )
 
-    # create a parsed selection from the select string
-    parsed_spec: SelectionSpec = graph_cli.parse_union([select], True)
-
-    if exclude:
-        parsed_spec = graph_cli.SelectionDifference(
-            components=[parsed_spec, graph_cli.parse_union([exclude], True)]
-        )
-
-    # execute this selection against the graph
-    selector = graph_selector.NodeSelector(graph, manifest)
-    selected, _ = selector.select_nodes(parsed_spec)
-    if len(selected) == 0:
-        raise DagsterInvalidSubsetError(f"No dbt models match the selection string '{select}'.")
-    return selected
-
 
 def _get_node_asset_key(node_info: Mapping[str, Any]) -> AssetKey:
-    """By default:
+    """Get the asset key for a dbt node.
 
+    By default:
         dbt sources: a dbt source's key is the union of its source name and its table name
         dbt models: a dbt model's key is the union of its model name and any schema configured on
     the model itself.
@@ -197,6 +168,7 @@ def _get_node_freshness_policy(node_info: Mapping[str, Any]) -> Optional[Freshne
         return FreshnessPolicy(
             maximum_lag_minutes=float(freshness_policy_config["maximum_lag_minutes"]),
             cron_schedule=freshness_policy_config.get("cron_schedule"),
+            cron_schedule_timezone=freshness_policy_config.get("cron_schedule_timezone"),
         )
     return None
 
@@ -230,13 +202,13 @@ def _get_deps(
             continue
 
         asset_deps[unique_id] = set()
-        for parent_unique_id in node_info["depends_on"]["nodes"]:
+        for parent_unique_id in node_info.get("depends_on", {}).get("nodes", []):
             parent_node_info = dbt_nodes[parent_unique_id]
             # for metrics or ephemeral dbt models, BFS to find valid parents
             if _is_non_asset_node(parent_node_info):
                 visited = set()
                 replaced_parent_ids = set()
-                queue = parent_node_info["depends_on"]["nodes"]
+                queue = parent_node_info.get("depends_on", {}).get("nodes", [])
                 while queue:
                     candidate_parent_id = queue.pop()
                     if candidate_parent_id in visited:
@@ -245,7 +217,7 @@ def _get_deps(
 
                     candidate_parent_info = dbt_nodes[candidate_parent_id]
                     if _is_non_asset_node(candidate_parent_info):
-                        queue.extend(candidate_parent_info["depends_on"]["nodes"])
+                        queue.extend(candidate_parent_info.get("depends_on", {}).get("nodes", []))
                     elif _valid_parent_node(candidate_parent_info):
                         replaced_parent_ids.add(candidate_parent_id)
 
@@ -267,6 +239,7 @@ def _get_asset_deps(
     node_info_to_asset_key,
     node_info_to_group_fn,
     node_info_to_freshness_policy_fn,
+    node_info_to_definition_metadata_fn,
     io_manager_key,
     display_raw_sql,
 ) -> Tuple[
@@ -303,12 +276,15 @@ def _get_asset_deps(
 
         asset_deps[asset_key] = set()
 
+        metadata = _get_node_metadata(node_info)
+        if node_info_to_definition_metadata_fn != _get_node_metadata:
+            metadata = merge_dicts(metadata, node_info_to_definition_metadata_fn(node_info))
         asset_outs[asset_key] = (
             output_name,
             Out(
                 io_manager_key=io_manager_key,
                 description=_get_node_description(node_info, display_raw_sql),
-                metadata=_get_node_metadata(node_info),
+                metadata=metadata,
                 is_required=False,
                 dagster_type=Nothing,
                 code_version=hashlib.sha1(
@@ -347,6 +323,144 @@ def _get_asset_deps(
     )
 
 
+def _batch_event_iterator(
+    context: OpExecutionContext,
+    dbt_resource: DbtCliResource,
+    use_build_command: bool,
+    node_info_to_asset_key: Callable[[Mapping[str, Any]], AssetKey],
+    runtime_metadata_fn: Optional[
+        Callable[[OpExecutionContext, Mapping[str, Any]], Mapping[str, RawMetadataValue]]
+    ],
+    kwargs: Dict[str, Any],
+) -> Iterator[Union[AssetObservation, AssetMaterialization, Output]]:
+    """Yields events for a dbt cli invocation. Waits until the entire command has completed before
+    emitting outputs.
+    """
+    dbt_output: Optional[DbtOutput] = None
+    try:
+        if use_build_command:
+            dbt_output = dbt_resource.build(**kwargs)
+        else:
+            dbt_output = dbt_resource.run(**kwargs)
+    finally:
+        # in the case that the project only partially runs successfully, still attempt to generate
+        # events for the parts that were successful
+        if dbt_output is None:
+            dbt_output = DbtOutput(result=check.not_none(dbt_resource.get_run_results_json()))
+
+        manifest_json = check.not_none(dbt_resource.get_manifest_json())
+
+        dbt_output = check.not_none(dbt_output)
+        for result in dbt_output.result["results"]:
+            extra_metadata: Optional[Mapping[str, RawMetadataValue]] = None
+            if runtime_metadata_fn:
+                node_info = manifest_json["nodes"][result["unique_id"]]
+                extra_metadata = runtime_metadata_fn(context, node_info)
+            yield from result_to_events(
+                result=result,
+                docs_url=dbt_output.docs_url,
+                node_info_to_asset_key=node_info_to_asset_key,
+                manifest_json=manifest_json,
+                extra_metadata=extra_metadata,
+                generate_asset_outputs=True,
+            )
+
+
+def _events_for_structured_json_line(
+    json_line: Mapping[str, Any],
+    context: OpExecutionContext,
+    node_info_to_asset_key: Callable[[Mapping[str, Any]], AssetKey],
+    runtime_metadata_fn: Optional[
+        Callable[[OpExecutionContext, Mapping[str, Any]], Mapping[str, RawMetadataValue]]
+    ],
+    manifest_json: Mapping[str, Any],
+) -> Iterator[Union[AssetObservation, Output]]:
+    """Parses a json line into a Dagster event. Attempts to replicate the behavior of result_to_events
+    as closely as possible.
+    """
+    runtime_node_info = json_line.get("data", {}).get("node_info", {})
+    if not runtime_node_info:
+        return
+
+    node_resource_type = runtime_node_info.get("resource_type")
+    node_status = runtime_node_info.get("node_status")
+    unique_id = runtime_node_info.get("unique_id")
+
+    if not node_resource_type or not unique_id:
+        return
+
+    compiled_node_info = manifest_json["nodes"][unique_id]
+
+    if node_resource_type in ASSET_RESOURCE_TYPES and node_status == "success":
+        metadata = dict(
+            runtime_metadata_fn(context, compiled_node_info) if runtime_metadata_fn else {}
+        )
+        started_at_str = runtime_node_info.get("node_started_at")
+        finished_at_str = runtime_node_info.get("node_finished_at")
+        if started_at_str is None or finished_at_str is None:
+            return
+
+        started_at = dateutil.parser.isoparse(started_at_str)  # type: ignore
+        completed_at = dateutil.parser.isoparse(finished_at_str)  # type: ignore
+        duration = completed_at - started_at
+        metadata.update(
+            {
+                "Execution Started At": started_at.isoformat(timespec="seconds"),
+                "Execution Completed At": completed_at.isoformat(timespec="seconds"),
+                "Execution Duration": duration.total_seconds(),
+            }
+        )
+        yield Output(
+            value=None,
+            output_name=_get_output_name(compiled_node_info),
+            metadata=metadata,
+        )
+    elif node_resource_type == "test" and runtime_node_info.get("node_finished_at") is not None:
+        upstream_unique_ids = (
+            manifest_json["nodes"][unique_id].get("depends_on", {}).get("nodes", [])
+        )
+        # tests can apply to multiple asset keys
+        for upstream_id in upstream_unique_ids:
+            # the upstream id can reference a node or a source
+            upstream_node_info = manifest_json["nodes"].get(upstream_id) or manifest_json[
+                "sources"
+            ].get(upstream_id)
+            if upstream_node_info is None:
+                continue
+            upstream_asset_key = node_info_to_asset_key(upstream_node_info)
+            yield AssetObservation(
+                asset_key=upstream_asset_key,
+                metadata={
+                    "Test ID": unique_id,
+                    "Test Status": node_status,
+                },
+            )
+
+
+def _stream_event_iterator(
+    context: OpExecutionContext,
+    dbt_resource: DbtCliResource,
+    use_build_command: bool,
+    node_info_to_asset_key: Callable[[Mapping[str, Any]], AssetKey],
+    runtime_metadata_fn: Optional[
+        Callable[[OpExecutionContext, Mapping[str, Any]], Mapping[str, RawMetadataValue]]
+    ],
+    kwargs: Dict[str, Any],
+) -> Iterator[Union[AssetObservation, Output]]:
+    """Yields events for a dbt cli invocation. Emits outputs as soon as the relevant dbt logs are
+    emitted.
+    """
+    manifest_json = check.not_none(dbt_resource.get_manifest_json())
+
+    for parsed_json_line in dbt_resource.cli_stream_json(
+        command="build" if use_build_command else "run",
+        **kwargs,
+    ):
+        yield from _events_for_structured_json_line(
+            parsed_json_line, context, node_info_to_asset_key, runtime_metadata_fn, manifest_json
+        )
+
+
 def _get_dbt_op(
     op_name: str,
     ins: Mapping[str, In],
@@ -387,59 +501,46 @@ def _get_dbt_op(
         ),
     )
     def _dbt_op(context):
-        dbt_output = None
-
         dbt_resource = getattr(context.resources, dbt_resource_key)
         # clean up any run results from the last run
         dbt_resource.remove_run_results_json()
 
+        kwargs: Dict[str, Any] = {}
         # in the case that we're running everything, opt for the cleaner selection string
         if len(context.selected_output_names) == len(outs):
-            kwargs = {"select": select, "exclude": exclude}
+            kwargs["select"] = select
+            kwargs["exclude"] = exclude
         else:
             # for each output that we want to emit, translate to a dbt select string by converting
             # the out to its corresponding fqn
-            kwargs = {
-                "select": [
-                    ".".join(fqns_by_output_name[output_name])
-                    for output_name in context.selected_output_names
-                ]
-            }
+            kwargs["select"] = [
+                ".".join(fqns_by_output_name[output_name])
+                for output_name in context.selected_output_names
+            ]
+        # variables to pass into the command
+        if partition_key_to_vars_fn:
+            kwargs["vars"] = partition_key_to_vars_fn(context.partition_key)
+        # merge in any additional kwargs from the config
+        kwargs = deep_merge_dicts(kwargs, context.op_config)
 
-        try:
-            # variables to pass into the command
-            if partition_key_to_vars_fn:
-                kwargs["vars"] = partition_key_to_vars_fn(context.partition_key)
-
-            # merge in any additional kwargs from the config
-            kwargs = deep_merge_dicts(kwargs, context.op_config)
-
-            if use_build_command:
-                dbt_output = dbt_resource.build(**kwargs)
-            else:
-                dbt_output = dbt_resource.run(**kwargs)
-        finally:
-            # in the case that the project only partially runs successfully, still attempt to generate
-            # events for the parts that were successful
-            if dbt_output is None:
-                dbt_output = DbtOutput(result=dbt_resource.get_run_results_json())
-
-            manifest_json = dbt_resource.get_manifest_json()
-
-            for result in dbt_output.result["results"]:
-                if runtime_metadata_fn:
-                    node_info = manifest_json["nodes"][result["unique_id"]]
-                    extra_metadata = runtime_metadata_fn(context, node_info)
-                else:
-                    extra_metadata = None
-                yield from result_to_events(
-                    result=result,
-                    docs_url=dbt_output.docs_url,
-                    node_info_to_asset_key=node_info_to_asset_key,
-                    manifest_json=manifest_json,
-                    extra_metadata=extra_metadata,
-                    generate_asset_outputs=True,
-                )
+        if _can_stream_events(dbt_resource):
+            yield from _stream_event_iterator(
+                context,
+                dbt_resource,
+                use_build_command,
+                node_info_to_asset_key,
+                runtime_metadata_fn,
+                kwargs,
+            )
+        else:
+            yield from _batch_event_iterator(
+                context,
+                dbt_resource,
+                use_build_command,
+                node_info_to_asset_key,
+                runtime_metadata_fn,
+                kwargs,
+            )
 
     return _dbt_op
 
@@ -463,6 +564,9 @@ def _dbt_nodes_to_assets(
     node_info_to_freshness_policy_fn: Callable[
         [Mapping[str, Any]], Optional[FreshnessPolicy]
     ] = _get_node_freshness_policy,
+    node_info_to_definition_metadata_fn: Callable[
+        [Mapping[str, Any]], Mapping[str, MetadataUserInput]
+    ] = _get_node_metadata,
     display_raw_sql: bool = True,
 ) -> AssetsDefinition:
     if use_build_command:
@@ -488,6 +592,7 @@ def _dbt_nodes_to_assets(
         node_info_to_asset_key=node_info_to_asset_key,
         node_info_to_group_fn=node_info_to_group_fn,
         node_info_to_freshness_policy_fn=node_info_to_freshness_policy_fn,
+        node_info_to_definition_metadata_fn=node_info_to_definition_metadata_fn,
         io_manager_key=io_manager_key,
         display_raw_sql=display_raw_sql,
     )
@@ -547,11 +652,13 @@ def load_assets_from_dbt_project(
     node_info_to_freshness_policy_fn: Callable[
         [Mapping[str, Any]], Optional[FreshnessPolicy]
     ] = _get_node_freshness_policy,
+    node_info_to_definition_metadata_fn: Callable[
+        [Mapping[str, Any]], Mapping[str, MetadataUserInput]
+    ] = _get_node_metadata,
     display_raw_sql: Optional[bool] = None,
     dbt_resource_key: str = "dbt",
 ) -> Sequence[AssetsDefinition]:
-    """
-    Loads a set of dbt models from a dbt project into Dagster assets.
+    """Loads a set of dbt models from a dbt project into Dagster assets.
 
     Creates one Dagster asset for each dbt model. All assets will be re-materialized using a single
     `dbt run` or `dbt build` command.
@@ -597,10 +704,13 @@ def load_assets_from_dbt_project(
             `dagster_freshness_policy={"maximum_lag_minutes": 60, "cron_schedule": "0 9 * * *"}`
             will result in that model being assigned
             `FreshnessPolicy(maximum_lag_minutes=60, cron_schedule="0 9 * * *")`
+        node_info_to_definition_metadata_fn (Dict[str, Any] -> Optional[Dict[str, MetadataUserInput]]):
+            A function that takes a dictionary of dbt node info and optionally returns a dictionary
+            of metadata to be attached to the corresponding definition. This is added to the default
+            metadata assigned to the node, which consists of the node's schema (if present).
         display_raw_sql (Optional[bool]): [Experimental] A flag to indicate if the raw sql associated
             with each model should be included in the asset description. For large projects, setting
             this flag to False is advised to reduce the size of the resulting snapshot.
-
     """
     project_dir = check.str_param(project_dir, "project_dir")
     profiles_dir = check.opt_str_param(
@@ -631,6 +741,7 @@ def load_assets_from_dbt_project(
         partition_key_to_vars_fn=partition_key_to_vars_fn,
         node_info_to_group_fn=node_info_to_group_fn,
         node_info_to_freshness_policy_fn=node_info_to_freshness_policy_fn,
+        node_info_to_definition_metadata_fn=node_info_to_definition_metadata_fn,
         display_raw_sql=display_raw_sql,
         dbt_resource_key=dbt_resource_key,
     )
@@ -655,11 +766,13 @@ def load_assets_from_dbt_manifest(
     node_info_to_freshness_policy_fn: Callable[
         [Mapping[str, Any]], Optional[FreshnessPolicy]
     ] = _get_node_freshness_policy,
+    node_info_to_definition_metadata_fn: Callable[
+        [Mapping[str, Any]], Mapping[str, MetadataUserInput]
+    ] = _get_node_metadata,
     display_raw_sql: Optional[bool] = None,
     dbt_resource_key: str = "dbt",
 ) -> Sequence[AssetsDefinition]:
-    """
-    Loads a set of dbt models, described in a manifest.json, into Dagster assets.
+    """Loads a set of dbt models, described in a manifest.json, into Dagster assets.
 
     Creates one Dagster asset for each dbt model. All assets will be re-materialized using a single
     `dbt run` command.
@@ -703,6 +816,10 @@ def load_assets_from_dbt_manifest(
             `dagster_freshness_policy={"maximum_lag_minutes": 60, "cron_schedule": "0 9 * * *"}`
             will result in that model being assigned
             `FreshnessPolicy(maximum_lag_minutes=60, cron_schedule="0 9 * * *")`
+        node_info_to_definition_metadata_fn (Dict[str, Any] -> Optional[Dict[str, MetadataUserInput]]):
+            A function that takes a dictionary of dbt node info and optionally returns a dictionary
+            of metadata to be attached to the corresponding definition. This is added to the default
+            metadata assigned to the node, which consists of the node's schema (if present).
         display_raw_sql (Optional[bool]): [Experimental] A flag to indicate if the raw sql associated
             with each model should be included in the asset description. For large projects, setting
             this flag to False is advised to reduce the size of the resulting snapshot.
@@ -719,6 +836,7 @@ def load_assets_from_dbt_manifest(
     if display_raw_sql is not None:
         experimental_arg_warning("display_raw_sql", "load_assets_from_dbt_manifest")
     display_raw_sql = check.opt_bool_param(display_raw_sql, "display_raw_sql", default=True)
+
     dbt_resource_key = check.str_param(dbt_resource_key, "dbt_resource_key")
 
     dbt_nodes = {
@@ -739,7 +857,11 @@ def load_assets_from_dbt_manifest(
         select = select if select is not None else "*"
         exclude = exclude if exclude is not None else ""
 
-        selected_unique_ids = _select_unique_ids_from_manifest_json(manifest_json, select, exclude)
+        selected_unique_ids = select_unique_ids_from_manifest(
+            select=select, exclude=exclude, manifest_json=manifest_json
+        )
+        if len(selected_unique_ids) == 0:
+            raise DagsterInvalidSubsetError(f"No dbt models match the selection string '{select}'.")
 
     dbt_assets_def = _dbt_nodes_to_assets(
         dbt_nodes,
@@ -756,6 +878,7 @@ def load_assets_from_dbt_manifest(
         partition_key_to_vars_fn=partition_key_to_vars_fn,
         node_info_to_group_fn=node_info_to_group_fn,
         node_info_to_freshness_policy_fn=node_info_to_freshness_policy_fn,
+        node_info_to_definition_metadata_fn=node_info_to_definition_metadata_fn,
         display_raw_sql=display_raw_sql,
     )
     dbt_assets: Sequence[AssetsDefinition]
