@@ -1,4 +1,4 @@
-from typing import Any, Mapping, Optional, Sequence, Set
+from typing import Any, Iterator, Mapping, Optional, Sequence, Set
 
 import dagster._check as check
 from dagster import Permissive, resource
@@ -8,13 +8,20 @@ from dagster._utils.merger import merge_dicts
 from ..dbt_resource import DbtResource
 from .constants import CLI_COMMON_FLAGS_CONFIG_SCHEMA, CLI_COMMON_OPTIONS_CONFIG_SCHEMA
 from .types import DbtCliOutput
-from .utils import execute_cli, parse_manifest, parse_run_results, remove_run_results
+from .utils import (
+    execute_cli,
+    execute_cli_stream,
+    parse_manifest,
+    parse_run_results,
+    remove_run_results,
+)
 
 
 class DbtCliResource(DbtResource):
-    """
-    A resource that allows you to execute dbt cli commands. For the most up-to-date documentation on
-    the specific parameters available to you for each command, check out the dbt docs:
+    """A resource that allows you to execute dbt cli commands.
+
+    For the most up-to-date documentation on the specific parameters available to you for each
+    command, check out the dbt docs:
 
     https://docs.getdbt.com/reference/commands/run
 
@@ -33,6 +40,7 @@ class DbtCliResource(DbtResource):
         docs_url: Optional[str] = None,
         json_log_format: bool = True,
         capture_logs: bool = True,
+        debug: bool = False,
     ):
         self._default_flags = default_flags
         self._executable = executable
@@ -42,27 +50,39 @@ class DbtCliResource(DbtResource):
         self._docs_url = docs_url
         self._json_log_format = json_log_format
         self._capture_logs = capture_logs
+        self._debug = debug
         super().__init__(logger)
 
     @property
     def default_flags(self) -> Mapping[str, Any]:
-        """
-        A set of params populated from resource config that are passed as flags to each dbt CLI command.
+        """A set of params populated from resource config that are passed as flags to each dbt CLI command.
         """
         return self._format_params(self._default_flags, replace_underscores=True)
 
     @property
     def strict_flags(self) -> Set[str]:
-        """
-        A set of flags that should not be auto-populated from the default flags unless they are
-            arguments to the associated function.
+        """A set of flags that should not be auto-populated from the default flags unless they are
+        arguments to the associated function.
         """
         return {"models", "exclude", "select"}
 
+    def _get_flags_dict(self, kwargs) -> Mapping[str, Any]:
+        extra_flags = {} if kwargs is None else kwargs
+
+        # remove default flags that are declared as "strict" and not explicitly passed in
+        default_flags = {
+            k: v
+            for k, v in self.default_flags.items()
+            if not (k in self.strict_flags and k not in extra_flags)
+        }
+
+        return merge_dicts(
+            default_flags, self._format_params(extra_flags, replace_underscores=True)
+        )
+
     @public
     def cli(self, command: str, **kwargs) -> DbtCliOutput:
-        """
-        Executes a dbt CLI command. Params passed in as keyword arguments will be merged with the
+        """Executes a dbt CLI command. Params passed in as keyword arguments will be merged with the
             default flags that were configured on resource initialization (if any) overriding the
             default values if necessary.
 
@@ -74,23 +94,10 @@ class DbtCliResource(DbtResource):
                 parsed log output as well as the contents of run_results.json (if applicable).
         """
         command = check.str_param(command, "command")
-        extra_flags = {} if kwargs is None else kwargs
-
-        # remove default flags that are declared as "strict" and not explicitly passed in
-        default_flags = {
-            k: v
-            for k, v in self.default_flags.items()
-            if not (k in self.strict_flags and k not in extra_flags)
-        }
-
-        flags = merge_dicts(
-            default_flags, self._format_params(extra_flags, replace_underscores=True)
-        )
-
         return execute_cli(
             executable=self._executable,
             command=command,
-            flags_dict=flags,
+            flags_dict=self._get_flags_dict(kwargs),
             log=self.logger,
             warn_error=self._warn_error,
             ignore_handled_error=self._ignore_handled_error,
@@ -98,8 +105,32 @@ class DbtCliResource(DbtResource):
             docs_url=self._docs_url,
             json_log_format=self._json_log_format,
             capture_logs=self._capture_logs,
+            debug=self._debug,
         )
 
+    def cli_stream_json(self, command: str, **kwargs) -> Iterator[Mapping[str, Any]]:
+        """Executes a dbt CLI command. Params passed in as keyword arguments will be merged with the
+            default flags that were configured on resource initialization (if any) overriding the
+            default values if necessary.
+
+        Args:
+            command (str): The command you wish to run (e.g. 'run', 'test', 'docs generate', etc.)
+        """
+        check.invariant(self._json_log_format, "Cannot stream JSON if json_log_format is False.")
+        for event in execute_cli_stream(
+            executable=self._executable,
+            command=command,
+            flags_dict=self._get_flags_dict(kwargs),
+            log=self.logger,
+            warn_error=self._warn_error,
+            ignore_handled_error=self._ignore_handled_error,
+            json_log_format=self._json_log_format,
+            capture_logs=self._capture_logs,
+            debug=self._debug,
+        ):
+            if event.parsed_json_line is not None:
+                yield event.parsed_json_line
+
     @public
     def compile(
         self,
@@ -108,8 +139,7 @@ class DbtCliResource(DbtResource):
         select: Optional[Sequence[str]] = None,
         **kwargs,
     ) -> DbtCliOutput:
-        """
-        Run the ``compile`` command on a dbt project. kwargs are passed in as additional parameters.
+        """Run the ``compile`` command on a dbt project. kwargs are passed in as additional parameters.
 
         Args:
             models (List[str], optional): the models to include in compilation.
@@ -130,8 +160,7 @@ class DbtCliResource(DbtResource):
         select: Optional[Sequence[str]] = None,
         **kwargs,
     ) -> DbtCliOutput:
-        """
-        Run the ``run`` command on a dbt project. kwargs are passed in as additional parameters.
+        """Run the ``run`` command on a dbt project. kwargs are passed in as additional parameters.
 
         Args:
             models (List[str], optional): the models to include in the run.
@@ -151,8 +180,7 @@ class DbtCliResource(DbtResource):
         exclude: Optional[Sequence[str]] = None,
         **kwargs,
     ) -> DbtCliOutput:
-        """
-        Run the ``snapshot`` command on a dbt project. kwargs are passed in as additional parameters.
+        """Run the ``snapshot`` command on a dbt project. kwargs are passed in as additional parameters.
 
         Args:
             select (List[str], optional): the snapshots to include in the run.
@@ -174,8 +202,7 @@ class DbtCliResource(DbtResource):
         select: Optional[Sequence[str]] = None,
         **kwargs,
     ) -> DbtCliOutput:
-        """
-        Run the ``test`` command on a dbt project. kwargs are passed in as additional parameters.
+        """Run the ``test`` command on a dbt project. kwargs are passed in as additional parameters.
 
         Args:
             models (List[str], optional): the models to include in testing.
@@ -211,8 +238,7 @@ class DbtCliResource(DbtResource):
         exclude: Optional[Sequence[str]] = None,
         **kwargs,
     ) -> DbtCliOutput:
-        """
-        Run the ``seed`` command on a dbt project. kwargs are passed in as additional parameters.
+        """Run the ``seed`` command on a dbt project. kwargs are passed in as additional parameters.
 
         Args:
             show (bool, optional): If ``True``, then show a sample of the seeded data in the
@@ -234,8 +260,7 @@ class DbtCliResource(DbtResource):
         exclude: Optional[Sequence[str]] = None,
         **kwargs,
     ) -> DbtCliOutput:
-        """
-        Run the ``ls`` command on a dbt project. kwargs are passed in as additional parameters.
+        """Run the ``ls`` command on a dbt project. kwargs are passed in as additional parameters.
 
         Args:
             select (List[str], optional): the resources to include in the output.
@@ -250,8 +275,7 @@ class DbtCliResource(DbtResource):
 
     @public
     def build(self, select: Optional[Sequence[str]] = None, **kwargs) -> DbtCliOutput:
-        """
-        Run the ``build`` command on a dbt project. kwargs are passed in as additional parameters.
+        """Run the ``build`` command on a dbt project. kwargs are passed in as additional parameters.
 
         Args:
             select (List[str], optional): the models/resources to include in the run.
@@ -264,8 +288,7 @@ class DbtCliResource(DbtResource):
 
     @public
     def freshness(self, select: Optional[Sequence[str]] = None, **kwargs) -> DbtCliOutput:
-        """
-        Run the ``source snapshot-freshness`` command on a dbt project. kwargs are passed in as additional parameters.
+        """Run the ``source snapshot-freshness`` command on a dbt project. kwargs are passed in as additional parameters.
 
         Args:
             select (List[str], optional): the sources to include in the run.
@@ -278,8 +301,7 @@ class DbtCliResource(DbtResource):
 
     @public
     def generate_docs(self, compile_project: bool = False, **kwargs) -> DbtCliOutput:
-        """
-        Run the ``docs generate`` command on a dbt project. kwargs are passed in as additional parameters.
+        """Run the ``docs generate`` command on a dbt project. kwargs are passed in as additional parameters.
 
         Args:
             compile_project (bool, optional): If true, compile the project before generating a catalog.
@@ -294,8 +316,7 @@ class DbtCliResource(DbtResource):
     def run_operation(
         self, macro: str, args: Optional[Mapping[str, Any]] = None, **kwargs
     ) -> DbtCliOutput:
-        """
-        Run the ``run-operation`` command on a dbt project. kwargs are passed in as additional parameters.
+        """Run the ``run-operation`` command on a dbt project. kwargs are passed in as additional parameters.
 
         Args:
             macro (str): the dbt macro to invoke.
@@ -309,8 +330,7 @@ class DbtCliResource(DbtResource):
 
     @public
     def get_run_results_json(self, **kwargs) -> Optional[Mapping[str, Any]]:
-        """
-        Get a parsed version of the run_results.json file for the relevant dbt project.
+        """Get a parsed version of the run_results.json file for the relevant dbt project.
 
         Returns:
             Dict[str, Any]: dictionary containing the parsed contents of the manifest json file
@@ -322,17 +342,14 @@ class DbtCliResource(DbtResource):
 
     @public
     def remove_run_results_json(self, **kwargs):
-        """
-        Remove the run_results.json file from previous runs (if it exists).
-        """
+        """Remove the run_results.json file from previous runs (if it exists)."""
         project_dir = kwargs.get("project_dir", self.default_flags["project-dir"])
         target_path = kwargs.get("target_path", self._target_path)
         remove_run_results(project_dir, target_path)
 
     @public
     def get_manifest_json(self, **kwargs) -> Optional[Mapping[str, Any]]:
-        """
-        Get a parsed version of the manifest.json file for the relevant dbt project.
+        """Get a parsed version of the manifest.json file for the relevant dbt project.
 
         Returns:
             Dict[str, Any]: dictionary containing the parsed contents of the manifest json file
@@ -369,4 +386,5 @@ def dbt_cli_resource(context) -> DbtCliResource:
         docs_url=context.resource_config.get("docs_url"),
         capture_logs=context.resource_config["capture_logs"],
         json_log_format=context.resource_config["json_log_format"],
+        debug=context.resource_config["debug"],
     )
