@@ -7,17 +7,25 @@ from typing import Any, Mapping, Optional, Sequence, cast
 from urllib.parse import urlencode, urljoin
 
 import requests
-from requests.exceptions import RequestException
-
-from dagster import Failure, Field, IntSource, MetadataValue, StringSource, __version__
-from dagster import _check as check
-from dagster import get_dagster_logger, resource
+from dagster import (
+    Failure,
+    Field,
+    IntSource,
+    MetadataValue,
+    StringSource,
+    __version__,
+    _check as check,
+    get_dagster_logger,
+    resource,
+)
 from dagster._utils.merger import deep_merge_dicts
+from requests.exceptions import RequestException
 
 from .types import DbtCloudOutput
 
 DBT_DEFAULT_HOST = "https://cloud.getdbt.com/"
-DBT_ACCOUNTS_PATH = "api/v2/accounts/"
+DBT_API_V2_PATH = "api/v2/accounts/"
+DBT_API_V3_PATH = "api/v3/accounts/"
 
 # default polling interval (in seconds)
 DEFAULT_POLL_INTERVAL = 10
@@ -32,7 +40,9 @@ class DbtCloudRunStatus(str, Enum):
     CANCELLED = "Cancelled"
 
 
-class DbtCloudResourceV2:
+# TODO: This resource should be a wrapper over an existing client for a accessing dbt Cloud,
+# rather than using requests to the API directly.
+class DbtCloudResource:
     """This class exposes methods on top of the dbt Cloud REST API v2.
 
     For a complete set of documentation on the dbt Cloud Administrative REST API, including expected
@@ -62,8 +72,12 @@ class DbtCloudResourceV2:
         self._log_requests = log_requests
 
     @property
-    def api_base_url(self) -> str:
-        return urljoin(self._dbt_cloud_host, DBT_ACCOUNTS_PATH)
+    def api_v2_base_url(self) -> str:
+        return urljoin(self._dbt_cloud_host, DBT_API_V2_PATH)
+
+    @property
+    def api_v3_base_url(self) -> str:
+        return urljoin(self._dbt_cloud_host, DBT_API_V3_PATH)
 
     def build_url_for_job(self, project_id: int, job_id: int) -> str:
         return urljoin(
@@ -82,27 +96,30 @@ class DbtCloudResourceV2:
         method: str,
         endpoint: str,
         data: Optional[Mapping[str, Any]] = None,
+        params: Optional[Mapping[str, Any]] = None,
         return_text: bool = False,
+        base_url: Optional[str] = None,
     ) -> Any:
-        """
-        Creates and sends a request to the desired dbt Cloud API endpoint.
+        """Creates and sends a request to the desired dbt Cloud API endpoint.
 
         Args:
             method (str): The http method to use for this request (e.g. "POST", "GET", "PATCH").
             endpoint (str): The dbt Cloud API endpoint to send this request to.
-            data (Optional[str]): JSON-formatted data string to be included in the request.
+            data (Optional[Mapping[str, Any]]): JSON-formatable data string to be included in the request.
+            params (Optional[Mapping[str, Any]]): Payload to add to query string of the request.
             return_text (bool): Override default behavior and return unparsed {"text": response.text}
                 blob instead of json.
+
         Returns:
             Dict[str, Any]: Parsed json data from the response to this request
         """
-
         headers = {
             "User-Agent": f"dagster-dbt/{__version__}",
             "Content-Type": "application/json",
             "Authorization": f"Bearer {self._auth_token}",
         }
-        url = urljoin(self.api_base_url, endpoint)
+        base_url = base_url or self.api_v2_base_url
+        url = urljoin(base_url, endpoint)
 
         if self._log_requests:
             self._log.debug(f"Making Request: method={method} url={url} data={data}")
@@ -115,7 +132,7 @@ class DbtCloudResourceV2:
                     url=url,
                     headers=headers,
                     data=json.dumps(data),
-                    allow_redirects=False,
+                    params=params,
                 )
                 response.raise_for_status()
                 return {"text": response.text} if return_text else response.json()["data"]
@@ -126,11 +143,33 @@ class DbtCloudResourceV2:
                 num_retries += 1
                 time.sleep(self._request_retry_delay)
 
-        raise Failure("Exceeded max number of retries.")
+        raise Failure(f"Max retries ({self._request_max_retries}) exceeded with url: {url}.")
+
+    def list_jobs(
+        self, project_id: int, order_by: Optional[str] = "-id"
+    ) -> Sequence[Mapping[str, Any]]:
+        """List all dbt jobs in a dbt Cloud project.
+
+        Args:
+            project_id (int): The ID of the relevant dbt Cloud project. You can find this value by
+                going to your account settings in the dbt Cloud UI. It will be the final
+                number in the url, e.g.: ``https://cloud.getdbt.com/next/settings/accounts/{account_id}/projects/{project_id}/``
+            order_by (Optional[str]): An identifier designated by dbt Cloud in which to sort the
+                results before returning them. Useful when combined with offset and limit to load
+                runs for a job. Defaults to "-id" where "-" designates reverse order and "id" is
+                the key to filter on.
+
+        Returns:
+            List[Dict[str, Any]]: Parsed json data from the response to this request
+        """
+        return self.make_request(
+            "GET",
+            f"{self._account_id}/jobs",
+            params={"project_id": project_id, "order_by": order_by},
+        )
 
     def get_job(self, job_id: int) -> Mapping[str, Any]:
-        """
-        Gets details about a given dbt job from the dbt Cloud API.
+        """Gets details about a given dbt job from the dbt Cloud API.
 
         Args:
             job_id (int): The ID of the relevant dbt Cloud job. You can find this value by going to
@@ -142,20 +181,21 @@ class DbtCloudResourceV2:
         return self.make_request("GET", f"{self._account_id}/jobs/{job_id}/")
 
     def update_job(self, job_id: int, **kwargs) -> Mapping[str, Any]:
-        """
-        Updates specific properties of a dbt job. Documentation on the full set of potential
-        parameters can be found here: https://docs.getdbt.com/dbt-cloud/api-v2#operation/updateJobById
+        """Updates specific properties of a dbt job.
+
+        Documentation on the full set of potential parameters can be found here:
+        https://docs.getdbt.com/dbt-cloud/api-v2#operation/updateJobById.
 
         Args:
             job_id (int): The ID of the relevant dbt Cloud job. You can find this value by going to
                 the details page of your job in the dbt Cloud UI. It will be the final number in the
                 url, e.g.: ``https://cloud.getdbt.com/#/accounts/{account_id}/projects/{project_id}/jobs/{job_id}/``
             kwargs: Passed in as the properties to be changed.
+
         Returns:
             Dict[str, Any]: Parsed json data from the response to this request
 
         Examples:
-
         .. code-block:: python
 
             # disable schedule for job with id=12345
@@ -169,10 +209,11 @@ class DbtCloudResourceV2:
         )
 
     def run_job(self, job_id: int, **kwargs) -> Mapping[str, Any]:
-        """
-        Initializes a run for a job. Overrides for specific properties can be set by passing in
-        values to the kwargs. A full list of overridable properties can be found here:
-        https://docs.getdbt.com/dbt-cloud/api-v2#operation/triggerRun
+        """Initializes a run for a job.
+
+        Overrides for specific properties can be set by passing in values to the kwargs. A full list
+        of overridable properties can be found here:
+        https://docs.getdbt.com/dbt-cloud/api-v2#operation/triggerRun.
 
         Args:
             job_id (int): The ID of the relevant dbt Cloud job. You can find this value by going to
@@ -183,13 +224,16 @@ class DbtCloudResourceV2:
         Returns:
             Dict[str, Any]: Parsed json data from the response to this request
         """
-        if self._disable_schedule_on_trigger:
-            self._log.info("Disabling dbt Cloud job schedule.")
-            self.update_job(job_id, triggers={"schedule": False})
         self._log.info(f"Initializing run for job with job_id={job_id}")
         if "cause" not in kwargs:
             kwargs["cause"] = "Triggered via Dagster"
         resp = self.make_request("POST", f"{self._account_id}/jobs/{job_id}/run/", data=kwargs)
+
+        has_schedule: bool = resp.get("job", {}).get("triggers", {}).get("schedule", False)
+        if has_schedule and self._disable_schedule_on_trigger:
+            self._log.info("Disabling dbt Cloud job schedule.")
+            self.update_job(job_id, triggers={"schedule": False})
+
         self._log.info(
             f"Run initialized with run_id={resp['id']}. View this run in "
             f"the dbt Cloud UI: {resp['href']}"
@@ -204,8 +248,7 @@ class DbtCloudResourceV2:
         offset: int = 0,
         limit: int = 100,
     ) -> Sequence[Mapping[str, object]]:
-        """
-        Returns a list of runs from dbt Cloud. This can be optionally filtered to a specific job
+        """Returns a list of runs from dbt Cloud. This can be optionally filtered to a specific job
         using the job_definition_id. It supports pagination using offset and limit as well and
         can be configured to load a variety of related information about the runs.
 
@@ -242,8 +285,7 @@ class DbtCloudResourceV2:
     def get_run(
         self, run_id: int, include_related: Optional[Sequence[str]] = None
     ) -> Mapping[str, Any]:
-        """
-        Gets details about a specific job run.
+        """Gets details about a specific job run.
 
         Args:
             run_id (int): The ID of the relevant dbt Cloud run. You can find this value by going to
@@ -263,8 +305,7 @@ class DbtCloudResourceV2:
         )
 
     def get_run_steps(self, run_id: int) -> Sequence[str]:
-        """
-        Gets the steps of an initialized dbt Cloud run.
+        """Gets the steps of an initialized dbt Cloud run.
 
         Args:
             run_id (int): The ID of the relevant dbt Cloud run. You can find this value by going to
@@ -280,8 +321,7 @@ class DbtCloudResourceV2:
         return steps_override or steps
 
     def cancel_run(self, run_id: int) -> Mapping[str, Any]:
-        """
-        Cancels a dbt Cloud run.
+        """Cancels a dbt Cloud run.
 
         Args:
             run_id (int): The ID of the relevant dbt Cloud run. You can find this value by going to
@@ -296,8 +336,7 @@ class DbtCloudResourceV2:
         return self.make_request("POST", f"{self._account_id}/runs/{run_id}/cancel/")
 
     def list_run_artifacts(self, run_id: int, step: Optional[int] = None) -> Sequence[str]:
-        """
-        Lists the paths of the available run artifacts from a completed dbt Cloud run.
+        """Lists the paths of the available run artifacts from a completed dbt Cloud run.
 
         Args:
             run_id (int): The ID of the relevant dbt Cloud run. You can find this value by going to
@@ -321,8 +360,7 @@ class DbtCloudResourceV2:
         )
 
     def get_run_artifact(self, run_id: int, path: str, step: Optional[int] = None) -> str:
-        """
-        The string contents of a run artifact from a dbt Cloud run.
+        """The string contents of a run artifact from a dbt Cloud run.
 
         Args:
             run_id (int): The ID of the relevant dbt Cloud run. You can find this value by going to
@@ -345,8 +383,7 @@ class DbtCloudResourceV2:
         )["text"]
 
     def get_manifest(self, run_id: int, step: Optional[int] = None) -> Mapping[str, Any]:
-        """
-        The parsed contents of a manifest.json file created by a completed run.
+        """The parsed contents of a manifest.json file created by a completed run.
 
         Args:
             run_id (int): The ID of the relevant dbt Cloud run. You can find this value by going to
@@ -362,8 +399,7 @@ class DbtCloudResourceV2:
         return json.loads(self.get_run_artifact(run_id, "manifest.json", step=step))
 
     def get_run_results(self, run_id: int, step: Optional[int] = None) -> Mapping[str, Any]:
-        """
-        The parsed contents of a run_results.json file created by a completed run.
+        """The parsed contents of a run_results.json file created by a completed run.
 
         Args:
             run_id (int): The ID of the relevant dbt Cloud run. You can find this value by going to
@@ -385,8 +421,7 @@ class DbtCloudResourceV2:
         poll_timeout: Optional[float] = None,
         href: Optional[str] = None,
     ) -> Mapping[str, Any]:
-        """
-        Polls a dbt Cloud job run until it completes. Will raise a `dagster.Failure` exception if the
+        """Polls a dbt Cloud job run until it completes. Will raise a `dagster.Failure` exception if the
         run does not complete successfully.
 
         Args:
@@ -404,7 +439,6 @@ class DbtCloudResourceV2:
             Dict[str, Any]: A dictionary containing the parsed contents of the dbt Cloud run details.
                 See: https://docs.getdbt.com/dbt-cloud/api-v2#operation/getRunById for schema.
         """
-
         status: Optional[str] = None
 
         if href is None:
@@ -435,8 +469,10 @@ class DbtCloudResourceV2:
                 ):
                     self.cancel_run(run_id)
                     raise Failure(
-                        f"Run {run_id} timed out after "
-                        f"{datetime.datetime.now() - poll_start}. Attempted to cancel.",
+                        (
+                            f"Run {run_id} timed out after "
+                            f"{datetime.datetime.now() - poll_start}. Attempted to cancel."
+                        ),
                         metadata={"run_page_url": MetadataValue.url(href)},
                     )
 
@@ -466,8 +502,7 @@ class DbtCloudResourceV2:
         poll_timeout: Optional[float] = None,
         **kwargs,
     ) -> DbtCloudOutput:
-        """
-        Runs a dbt Cloud job and polls until it completes. Will raise a `dagster.Failure` exception
+        """Runs a dbt Cloud job and polls until it completes. Will raise a `dagster.Failure` exception
         if the run does not complete successfully.
 
         Args:
@@ -493,7 +528,7 @@ class DbtCloudResourceV2:
         try:
             run_results = self.get_run_results(run_id)
         # if you fail to get run_results for this job, just leave it empty
-        except Failure as _:
+        except Failure:
             self._log.info(
                 "run_results.json not available for this run. Defaulting to empty value."
             )
@@ -503,34 +538,95 @@ class DbtCloudResourceV2:
             self._log.info(f"Docs for this run can be viewed here: {output.docs_url}")
         return output
 
+    def get_job_environment_variables(self, project_id: int, job_id: int) -> Mapping[str, Any]:
+        """Get the dbt Cloud environment variables for a specific job.
+
+        Args:
+            project_id (int): The ID of the relevant dbt Cloud project. You can find this value by
+                going to your account settings in the dbt Cloud UI. It will be the final
+                number in the url, e.g.: ``https://cloud.getdbt.com/next/settings/accounts/{account_id}/projects/{project_id}/``
+            job_id (int): The ID of the relevant dbt Cloud job. You can find this value by going to
+                the details page of your job in the dbt Cloud UI. It will be the final number in the
+                url, e.g.: ``https://cloud.getdbt.com/#/accounts/{account_id}/projects/{project_id}/jobs/{job_id}/``
+        """
+        return self.make_request(
+            "GET",
+            f"{self._account_id}/projects/{project_id}/environment-variables/job",
+            params={"job_definition_id": job_id},
+            base_url=self.api_v3_base_url,
+        )
+
+    def set_job_environment_variable(
+        self, project_id: int, job_id: int, environment_variable_id: int, name: str, value: str
+    ) -> Mapping[str, Any]:
+        """Set the dbt Cloud environment variables for a specific job.
+
+        Args:
+            project_id (int): The ID of the relevant dbt Cloud project. You can find this value by
+                going to your account settings in the dbt Cloud UI. It will be the final
+                number in the url, e.g.: ``https://cloud.getdbt.com/next/settings/accounts/{account_id}/projects/{project_id}/``
+            job_id (int): The ID of the relevant dbt Cloud job. You can find this value by going to
+                the details page of your job in the dbt Cloud UI. It will be the final number in the
+                url, e.g.: ``https://cloud.getdbt.com/#/accounts/{account_id}/projects/{project_id}/jobs/{job_id}/``
+            name (str): The name of the environment variable to set.
+            value (str): The raw value of the environment variable.
+        """
+        return self.make_request(
+            "POST",
+            f"{self._account_id}/projects/{project_id}/environment-variables/{environment_variable_id}",
+            data={
+                "id": environment_variable_id,
+                "account_id": self._account_id,
+                "project_id": project_id,
+                "job_definition_id": job_id,
+                "type": "job",
+                "name": name,
+                "raw_value": value,
+            },
+            base_url=self.api_v3_base_url,
+        )
+
+
+# This is a temporary shim to support the old resource name.
+DbtCloudResourceV2 = DbtCloudResource
+
 
 @resource(
     config_schema={
         "auth_token": Field(
             StringSource,
             is_required=True,
-            description="dbt Cloud API Token. User tokens can be found in the "
-            "[dbt Cloud UI](https://cloud.getdbt.com/#/profile/api/), or see the "
-            "[dbt Cloud Docs](https://docs.getdbt.com/docs/dbt-cloud/dbt-cloud-api/service-tokens) "
-            "for instructions on creating a Service Account token.",
+            description=(
+                "dbt Cloud API Token. User tokens can be found in the [dbt Cloud"
+                " UI](https://cloud.getdbt.com/#/profile/api/), or see the [dbt Cloud"
+                " Docs](https://docs.getdbt.com/docs/dbt-cloud/dbt-cloud-api/service-tokens) for"
+                " instructions on creating a Service Account token."
+            ),
         ),
         "account_id": Field(
             IntSource,
             is_required=True,
-            description="dbt Cloud Account ID. This value can be found in the url of a variety of "
-            "views in the dbt Cloud UI, e.g. https://cloud.getdbt.com/#/accounts/{account_id}/settings/.",
+            description=(
+                "dbt Cloud Account ID. This value can be found in the url of a variety of views in"
+                " the dbt Cloud UI, e.g."
+                " https://cloud.getdbt.com/#/accounts/{account_id}/settings/."
+            ),
         ),
         "disable_schedule_on_trigger": Field(
             bool,
             default_value=True,
-            description="Specifies if you would like any job that is triggered using this "
-            "resource to automatically disable its schedule.",
+            description=(
+                "Specifies if you would like any job that is triggered using this "
+                "resource to automatically disable its schedule."
+            ),
         ),
         "request_max_retries": Field(
             int,
             default_value=3,
-            description="The maximum number of times requests to the dbt Cloud API should be retried "
-            "before failing.",
+            description=(
+                "The maximum number of times requests to the dbt Cloud API should be retried "
+                "before failing."
+            ),
         ),
         "request_retry_delay": Field(
             float,
@@ -540,14 +636,16 @@ class DbtCloudResourceV2:
         "dbt_cloud_host": Field(
             config=StringSource,
             default_value=DBT_DEFAULT_HOST,
-            description="The hostname where dbt cloud is being hosted (e.g. https://my_org.cloud.getdbt.com/).",
+            description=(
+                "The hostname where dbt cloud is being hosted (e.g."
+                " https://my_org.cloud.getdbt.com/)."
+            ),
         ),
     },
     description="This resource helps interact with dbt Cloud connectors",
 )
-def dbt_cloud_resource(context) -> DbtCloudResourceV2:
-    """
-    This resource allows users to programatically interface with the dbt Cloud Administrative REST
+def dbt_cloud_resource(context) -> DbtCloudResource:
+    """This resource allows users to programatically interface with the dbt Cloud Administrative REST
     API (v2) to launch jobs and monitor their progress. This currently implements only a subset of
     the functionality exposed by the API.
 
@@ -575,7 +673,7 @@ def dbt_cloud_resource(context) -> DbtCloudResourceV2:
         def my_dbt_cloud_job():
             ...
     """
-    return DbtCloudResourceV2(
+    return DbtCloudResource(
         auth_token=context.resource_config["auth_token"],
         account_id=context.resource_config["account_id"],
         disable_schedule_on_trigger=context.resource_config["disable_schedule_on_trigger"],
