@@ -1,9 +1,26 @@
-from typing import Any, Callable, Dict, Iterator, Mapping, Optional, Sequence, Union, cast
+from pathlib import Path
+from typing import (
+    AbstractSet,
+    Any,
+    Callable,
+    Dict,
+    Iterator,
+    Mapping,
+    Optional,
+    Sequence,
+    Union,
+    cast,
+)
 
 import dateutil
-
-from dagster import AssetKey, AssetMaterialization, AssetObservation, MetadataValue, Output
-from dagster import _check as check
+from dagster import (
+    AssetKey,
+    AssetMaterialization,
+    AssetObservation,
+    MetadataValue,
+    Output,
+    _check as check,
+)
 from dagster._core.definitions.metadata import RawMetadataValue
 
 from .types import DbtOutput
@@ -72,8 +89,7 @@ def result_to_events(
     extra_metadata: Optional[Mapping[str, RawMetadataValue]] = None,
     generate_asset_outputs: bool = False,
 ) -> Iterator[Union[AssetMaterialization, AssetObservation, Output]]:
-    """
-    This is a hacky solution that attempts to consolidate parsing many of the potential formats
+    """This is a hacky solution that attempts to consolidate parsing many of the potential formats
     that dbt can provide its results in. This is known to work for CLI Outputs for dbt versions 0.18+,
     as well as RPC responses for a similar time period, but as the RPC response schema is not documented
     nor enforced, this can become out of date easily.
@@ -119,7 +135,6 @@ def result_to_events(
     node_resource_type = _resource_type(unique_id)
 
     if node_resource_type in ASSET_RESOURCE_TYPES and status == "success":
-
         if generate_asset_outputs:
             yield Output(
                 value=None,
@@ -133,7 +148,7 @@ def result_to_events(
                 metadata=metadata,
             )
     # can only associate tests with assets if we have manifest_json available
-    elif node_resource_type == "test" and manifest_json:
+    elif node_resource_type == "test" and manifest_json and status != "skipped":
         upstream_unique_ids = manifest_json["nodes"][unique_id]["depends_on"]["nodes"]
         # tests can apply to multiple asset keys
         for upstream_id in upstream_unique_ids:
@@ -159,14 +174,11 @@ def generate_events(
     node_info_to_asset_key: Optional[Callable[[Mapping[str, Any]], AssetKey]] = None,
     manifest_json: Optional[Mapping[str, Any]] = None,
 ) -> Iterator[Union[AssetMaterialization, AssetObservation]]:
-
-    """
-    This function yields :py:class:`dagster.AssetMaterialization` events for each model updated by
+    """This function yields :py:class:`dagster.AssetMaterialization` events for each model updated by
     a dbt command, and :py:class:`dagster.AssetObservation` events for each test run.
 
     Information parsed from a :py:class:`~DbtOutput` object.
     """
-
     for result in dbt_output.result["results"]:
         for event in result_to_events(
             result,
@@ -184,8 +196,7 @@ def generate_materializations(
     dbt_output: DbtOutput,
     asset_key_prefix: Optional[Sequence[str]] = None,
 ) -> Iterator[AssetMaterialization]:
-    """
-    This function yields :py:class:`dagster.AssetMaterialization` events for each model updated by
+    """This function yields :py:class:`dagster.AssetMaterialization` events for each model updated by
     a dbt command.
 
     Information parsed from a :py:class:`~DbtOutput` object.
@@ -195,28 +206,27 @@ def generate_materializations(
     `dbt_rpc_sync_resource`, which will wait for execution to complete.
 
     Examples:
+        .. code-block:: python
 
-    .. code-block:: python
+            from dagster import op, Output
+            from dagster_dbt.utils import generate_materializations
+            from dagster_dbt import dbt_cli_resource, dbt_rpc_sync_resource
 
-        from dagster import op, Output
-        from dagster_dbt.utils import generate_materializations
-        from dagster_dbt import dbt_cli_resource, dbt_rpc_sync_resource
+            @op(required_resource_keys={"dbt"})
+            def my_custom_dbt_run(context):
+                dbt_output = context.resources.dbt.run()
+                for materialization in generate_materializations(dbt_output):
+                    # you can modify the materialization object to add extra metadata, if desired
+                    yield materialization
+                yield Output(my_dbt_output)
 
-        @op(required_resource_keys={"dbt"})
-        def my_custom_dbt_run(context):
-            dbt_output = context.resources.dbt.run()
-            for materialization in generate_materializations(dbt_output):
-                # you can modify the materialization object to add extra metadata, if desired
-                yield materialization
-            yield Output(my_dbt_output)
+            @job(resource_defs={{"dbt":dbt_cli_resource}})
+            def my_dbt_cli_job():
+                my_custom_dbt_run()
 
-        @job(resource_defs={{"dbt":dbt_cli_resource}})
-        def my_dbt_cli_job():
-            my_custom_dbt_run()
-
-        @job(resource_defs={{"dbt":dbt_rpc_sync_resource}})
-        def my_dbt_rpc_job():
-            my_custom_dbt_run()
+            @job(resource_defs={{"dbt":dbt_rpc_sync_resource}})
+            def my_dbt_rpc_job():
+                my_custom_dbt_run()
     """
     asset_key_prefix = check.opt_sequence_param(asset_key_prefix, "asset_key_prefix", of_type=str)
 
@@ -227,3 +237,85 @@ def generate_materializations(
         ),
     ):
         yield check.inst(cast(AssetMaterialization, event), AssetMaterialization)
+
+
+def select_unique_ids_from_manifest(
+    select: str,
+    exclude: str,
+    state_path: Optional[str] = None,
+    manifest_json_path: Optional[str] = None,
+    manifest_json: Optional[Mapping[str, Any]] = None,
+) -> AbstractSet[str]:
+    """Method to apply a selection string to an existing manifest.json file."""
+    try:
+        import dbt.flags as flags
+        import dbt.graph.cli as graph_cli
+        import dbt.graph.selector as graph_selector
+        from dbt.contracts.graph.manifest import Manifest, WritableManifest
+        from dbt.contracts.state import PreviousState
+        from dbt.graph import SelectionSpec
+        from dbt.graph.selector_spec import IndirectSelection
+        from networkx import DiGraph
+
+    except ImportError as e:
+        raise check.CheckError(
+            "In order to use the `select` argument on load_assets_from_dbt_manifest, you must have"
+            "`dbt-core >= 1.0.0` and `networkx` installed."
+        ) from e
+
+    if state_path is not None:
+        previous_state = PreviousState(
+            path=Path(state_path),
+            current_path=Path("/tmp/null")
+            if manifest_json_path is None
+            else Path(manifest_json_path),
+        )
+    else:
+        previous_state = None
+
+    if manifest_json_path is not None:
+        manifest = WritableManifest.read_and_check_versions(manifest_json_path)
+        child_map = manifest.child_map
+    elif manifest_json is not None:
+
+        class _DictShim(dict):
+            """Shim to enable hydrating a dictionary into a dot-accessible object."""
+
+            def __getattr__(self, item):
+                ret = super().get(item)
+                # allow recursive access e.g. foo.bar.baz
+                return _DictShim(ret) if isinstance(ret, dict) else ret
+
+        manifest = Manifest(
+            # dbt expects dataclasses that can be accessed with dot notation, not bare dictionaries
+            nodes={
+                unique_id: _DictShim(info) for unique_id, info in manifest_json["nodes"].items()
+            },
+            sources={
+                unique_id: _DictShim(info) for unique_id, info in manifest_json["sources"].items()
+            },
+            metrics={
+                unique_id: _DictShim(info) for unique_id, info in manifest_json["metrics"].items()
+            },
+            exposures={
+                unique_id: _DictShim(info) for unique_id, info in manifest_json["exposures"].items()
+            },
+        )
+        child_map = manifest_json["child_map"]
+    else:
+        check.failed("Must provide either a manifest_json_path or manifest_json.")
+    graph = graph_selector.Graph(DiGraph(incoming_graph_data=child_map))
+
+    # create a parsed selection from the select string
+    flags.INDIRECT_SELECTION = IndirectSelection.Eager
+    parsed_spec: SelectionSpec = graph_cli.parse_union([select], True)
+
+    if exclude:
+        parsed_spec = graph_cli.SelectionDifference(
+            components=[parsed_spec, graph_cli.parse_union([exclude], True)]
+        )
+
+    # execute this selection against the graph
+    selector = graph_selector.NodeSelector(graph, manifest, previous_state=previous_state)
+    selected, _ = selector.select_nodes(parsed_spec)
+    return selected
